--- a/drivers/reset/reset-ath79.c
+++ b/drivers/reset/reset-ath79.c
@@ -12,6 +12,7 @@
 #include <linux/platform_device.h>
 #include <linux/reset-controller.h>
 #include <linux/reboot.h>
+#include <linux/delay.h>
 
 struct ath79_reset {
 	struct reset_controller_dev rcdev;
@@ -21,16 +22,13 @@ struct ath79_reset {
 };
 
 #define FULL_CHIP_RESET 24
+#define ETH_SWITCH_RESET 8
+#define ETH_SWITCH_ARESET 12
 
-static int ath79_reset_update(struct reset_controller_dev *rcdev,
+static void __ath79_reset_update_unlocked(struct ath79_reset *ath79_reset,
 			unsigned long id, bool assert)
 {
-	struct ath79_reset *ath79_reset =
-		container_of(rcdev, struct ath79_reset, rcdev);
-	unsigned long flags;
 	u32 val;
-
-	spin_lock_irqsave(&ath79_reset->lock, flags);
 	val = readl(ath79_reset->base);
 	if (assert)
 		val |= BIT(id);
@@ -39,6 +37,17 @@ static int ath79_reset_update(struct res
 	writel(val, ath79_reset->base);
 	/* Flush cache */
 	readl(ath79_reset->base);
+}
+
+static int ath79_reset_update(struct reset_controller_dev *rcdev,
+			unsigned long id, bool assert)
+{
+	struct ath79_reset *ath79_reset =
+		container_of(rcdev, struct ath79_reset, rcdev);
+	unsigned long flags;
+
+	spin_lock_irqsave(&ath79_reset->lock, flags);
+	__ath79_reset_update_unlocked(ath79_reset, id, assert);
 	spin_unlock_irqrestore(&ath79_reset->lock, flags);
 
 	return 0;
@@ -79,8 +88,16 @@ static int ath79_reset_restart_handler(s
 {
 	struct ath79_reset *ath79_reset =
 		container_of(nb, struct ath79_reset, restart_nb);
+	unsigned long flags;
 
-	ath79_reset_assert(&ath79_reset->rcdev, FULL_CHIP_RESET);
+	spin_lock_irqsave(&ath79_reset->lock, flags);
+	__ath79_reset_update_unlocked(ath79_reset, ETH_SWITCH_RESET, true);
+	mdelay(2);
+	__ath79_reset_update_unlocked(ath79_reset, ETH_SWITCH_ARESET, true);
+	mdelay(2);
+	__ath79_reset_update_unlocked(ath79_reset, FULL_CHIP_RESET, true);
+	mdelay(10);
+	spin_unlock_irqrestore(&ath79_reset->lock, flags);
 
 	return NOTIFY_DONE;
 }
